#include <fs_attr.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include <Node.h>
#include <Alert.h>
#include <File.h>
#include <Application.h>
#include <Button.h>
#include <StringView.h>
#include <TextControl.h>
#include <StringView.h>
#include <CheckBox.h>
#include <ScrollView.h>
#include <Menu.h>
#include <MenuItem.h>
#include <MenuField.h>
#include <Volume.h>
#include <Directory.h>
#include <Path.h>
#include <FindDirectory.h>
#include <E-mail.h>

#include "FilelistItem.h"
#include "FilelistView.h"
#include "MailToWindow.h"

CMailToWindow::CMailToWindow()
	: BWindow( BRect( 200, 100, 480, 300 ), "Mail To", B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE )
{
	Build();
}

CMailToWindow::CMailToWindow( float x, float y )
	: BWindow( BRect( x, y, x+200, y+200 ), "Mail To", B_TITLED_WINDOW, 0 )
{
#ifdef DEBUG
	printf( "::CMailToWindow\n" );
#endif
	Build();
}

CMailToWindow::~CMailToWindow()
{
#ifdef DEBUG
	printf( "::~CMailToWindow\n" );
#endif
	CFilelistItem *item;

	delete[] _zipstring;
	delete[] _zipname;
	
	for( int32 i = 0; ( item = ( CFilelistItem * )_Listview->ItemAt( i ) ); i++ )
		delete item;
}

bool CMailToWindow::QuitRequested()
{
#ifdef DEBUG
	printf( "::QuitRequested\n" );
#endif
	if( isApp )
		be_app->PostMessage( B_QUIT_REQUESTED );
	return true;
}

void CMailToWindow::Build()
{
#ifdef DEBUG
	printf( "::Build\n" );
#endif

    char dateString[ 16 ] = "";

	isApp = false;

	BMenuField *field;
	BRect r = Bounds();
	r.InsetBy( -1, -1 );
	
	time_t          now;
	struct tm       *date;
	now = time(NULL);
	date = localtime(&now);
	strftime(dateString, 16, "_%D%T", date);

	_zipstring = new char[ B_FILE_NAME_LENGTH*2+128 ];  // some black magic number from R'alf's ZipMe
	_zipname = new char[ B_FILE_NAME_LENGTH*2+128 ];  // some black magic number from R'alf's ZipMe
	
	BDirectory dir;
	if( dir.SetTo( "/boot/home/temp/" ) == B_OK )
		sprintf( _zipname, "/boot/home/temp/zipmail%s.zip", dateString );
	else
		sprintf( _zipname, "/boot/home/zipmail%s.zip", dateString );
	strcpy( _zipstring, "zip -ry " );
	strcat( _zipstring, _zipname );
	strcat( _zipstring, " " );
	
	_PanelView = new CPanelView( Bounds(), "back", B_FOLLOW_ALL, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE | B_DRAW_ON_CHILDREN );
	AddChild( _PanelView );

	_SendToMenu = new BPopUpMenu( "To:" );
	_SendToMenu->SetRadioMode( false );
	BuildMenu();

	field = new BMenuField( BRect( 10, 6, 14, 22 ), "", NULL, _SendToMenu );
	_PanelView->AddChild( field );

	BStringView *v = new BStringView( BRect( 6, 6, 6 + 20, 22 ), "name", "To:" );
	v->SetAlignment( B_ALIGN_RIGHT );
	//_PanelView->AddChild( v );	

	_Email = new BTextControl( BRect( 48 , 8, 270, 22 ), "email", NULL, "", new BMessage('emil') );
	_PanelView->AddChild( _Email );

	_Subject = new BTextControl( BRect( 48 , 32, 270, 38 ), "subjectl", NULL, "", new BMessage('subj') );
	_PanelView->AddChild( _Subject );
	_Subject->SetTarget( this );

	_Listview = new CFilelistView( BRect( 53, 58, 267-B_V_SCROLL_BAR_WIDTH, 160 ), "list" );
	BScrollView *sv = new BScrollView( "scroll", _Listview, B_FOLLOW_LEFT | B_FOLLOW_TOP, 0, false, true );
	_PanelView->AddChild( sv );

	_isZipped = new BCheckBox( BRect( 50, 167, 167, 167+10 ), "zipped", "Put in zip archive", NULL );
	_PanelView->AddChild( _isZipped );
	_isZipped->SetTarget( this );
	_isZipped->SetValue( B_CONTROL_ON );

	_SendButton = new BButton( BRect( 199, 167, 199+24+48, 167+24 ), "sendb", "Send", new BMessage( SEND_BUTTON ) );
	_PanelView->AddChild( _SendButton );
	_SendButton->SetTarget( this );

	BStringView *tv = new BStringView( BRect( 6, 32, 44, 48 ), "text", "Subject:" );
	_PanelView->AddChild( tv );

}

void CMailToWindow::BuildMenu()
{
#ifdef DEBUG
	printf( "::BuildMenu()\n" );
#endif
	BMenuItem 	*item;
	BEntry		entry;
	BPath 		path;
   	BDirectory	dir;
	BMessage	*msg;
	BFile		file;
	attr_info	ainfo;
	char 		*txt;		
	char		name[B_FILE_NAME_LENGTH]; 

	entry_ref	e;

	find_directory( B_USER_DIRECTORY, &path );
	path.Append( "people" );

   	status_t s = dir.SetTo( path.Path() );
	if( s == B_ENTRY_NOT_FOUND )
		return;

	txt = (char *)malloc(1);
	txt[0] = 0;
	
	_SendToMenu->AddSeparatorItem();
	while( dir.GetNextEntry( &entry ) != B_ENTRY_NOT_FOUND )
	{
		if (entry.InitCheck() == B_OK) 
		{
			entry.GetPath( &path );
			entry.GetName( name );
			if (path.InitCheck() == B_OK) 
			{
				file.SetTo( &entry, B_READ_ONLY );
				if( file.GetAttrInfo( "META:email", &ainfo ) == B_NO_ERROR )
				{
					if( ainfo.size > 1 )
					{
						txt = (char *)realloc( txt, ainfo.size );
						file.ReadAttr( "META:email", B_STRING_TYPE, 0, txt, ainfo.size );
						msg = new BMessage( MENU_PERSON );
						msg->AddString( "email", txt );
						item = new BMenuItem( name, msg );
						
						file.GetAttrInfo( "META:group", &ainfo );
						if( ainfo.size > 1 )
						{
							txt = (char *)realloc( txt, ainfo.size );
							file.ReadAttr( "META:group", B_STRING_TYPE, 0, txt, ainfo.size );
							BMenuItem *mi = _SendToMenu->FindItem( txt );
							if( mi != NULL )
							{
								BMenu *m = mi->Submenu();
								m->AddItem( item );
							}
							else
							{
								BMenu *m = new BMenu( txt );
								m->AddItem( item ); 
								_SendToMenu->AddItem( m, 0 );
							}
//							AddItem( txt, item );
						}
						else
						{
							_SendToMenu->AddItem( item, _SendToMenu->CountItems() );
						}
					}
				}
			}
		}
	}
#ifdef DEBUG
	printf( "::BuildMenuEnd\n" );	
#endif
}

void CMailToWindow::MessageReceived( BMessage *msg )
{
#ifdef DEBUG
	printf( "::MessageReceived\n" );
#endif
	switch( msg->what )
	{
		case MENU_PERSON:
		{
			const char *c = msg->FindString( "email" );
			_Email->SetText( c );
		}
			break;
		case B_SIMPLE_DATA:
		{
			entry_ref ref;
			for ( int count = 0; msg->FindRef( "refs", count, &ref ) == B_NO_ERROR; count ++ ) 
			{
				AddRef( &ref );
			}
		}
			break;		
		case SEND_BUTTON:
		{
			if( strlen( _Email->Text() ) < 3 )
			{
				(new BAlert( "", "Bad E-mail adress", "OK" ) )->Go();
				_Email->MakeFocus();
				break;
			}
			if( strlen( _Subject->Text() ) == 0 )
			{
				(new BAlert( "", "No subject", "OK" ) )->Go();
				_Subject->MakeFocus();
				break;
			}	

			CFilelistItem *tmp;
			if( _isZipped->Value() == B_CONTROL_OFF )
			{
				BNode node;
				bool bfolder = false;
				for( int i = 0; i < _Listview->CountItems(); i++ )
				{
					tmp = ( CFilelistItem * )_Listview->ItemAt( i );
					node.SetTo( tmp->Ref() );
					if( node.IsDirectory() )
						bfolder = true;
				}
				if( bfolder )
				{
					(new BAlert( "", "Can not mail unzipped folders", "OK" ) )->Go();
					break;
				}
			}
			
			BMailMessage *m = new BMailMessage();
			m->AddHeaderField( B_MAIL_TO, _Email->Text() );
			m->AddHeaderField( B_MAIL_SUBJECT, _Subject->Text() );
			BEntry en;
			BPath p;
			for( int i = 0; i < _Listview->CountItems(); i++ )
			{
				tmp = ( CFilelistItem * )_Listview->ItemAt( i );
				if( _isZipped->Value() == B_CONTROL_ON )
				{
					en.SetTo( tmp->Ref() );
					en.GetPath( &p );
					strcat( _zipstring, p.Path() );
					strcat( _zipstring, " " );
				}
				else
					m->AddEnclosure( tmp->Ref() );
			}
			if( _isZipped->Value() == B_CONTROL_ON )
			{
				if( system( _zipstring ) != 0 )
  				{
  					(new BAlert( "", "Something wen horribly wrong!", "OK" ) )->Go();
  					(new BAlert( "", _zipstring, "OK" ) )->Go();
  					exit(0);
  				}
				m->AddEnclosure( _zipname );
			}
			m->Send( true );
			PostMessage( B_QUIT_REQUESTED );
			BEntry e( _zipname );
			e.Remove();
		}
			break;
		default:	
			BWindow::MessageReceived( msg );
	}
}

void CMailToWindow::AddRef( entry_ref *ref )
{
#ifdef DEBUG
	printf( "::AddRef\n" );
#endif
	BNode node;
	node.SetTo( ref );
	if ( node.InitCheck( ) == B_OK ) 
	{
		BEntry e;
		BPath p;
		e.SetTo( ref );
		e.GetPath( &p );

		bool inlist = false;
		CFilelistItem *tmp;
		CFilelistItem *item = new CFilelistItem( ref );

		Looper()->Lock();
		for( int i = 0; i < _Listview->CountItems(); i++ )
		{
			tmp = ( CFilelistItem * )_Listview->ItemAt( i );
			if( *tmp == *item )
				inlist = true;
		}
		if( !inlist )
		{
			_Listview->AddItem( item );
		}
		Looper()->Unlock();
	}
}

rgb_color rgb( uint8 r, uint8 g, uint8 b )
{
	rgb_color c;
	c.red = r;
	c.green = g;
	c.blue = b;
	return c;
}

